package ubisec.net;

import java.net.InetAddress;
import ubisec.util.BitSet2;

/**
*
* This class represents an Internet Protocol (IP) address. 
* <p>
* An IP address is either a 32-bit or 128-bit unsigned number used by IP, a lower-level protocol on which protocols
like UDP and TCP are built. The IP address architecture is defined by RFC 790: Assigned Numbers, RFC 1918:
Address Allocation for Private Internets, RFC 2365: Administratively Scoped IP Multicast, and RFC 2373: IP
Version 6 Addressing Architecture.An instance of an IPAddress consists of an array of BitSet2 that contains the bits set of an IP address presentation
 
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 oct. 2004
*
* 
*  
*/

public class IPAddress

{
    BitSet2[] address = new BitSet2[4];
    
    /**
	*
	* Creates an empty IPAddress object. IP address equals 0.0.0.0
	*  
	* @see ubisec.net.IPAddress#IPAddress(BitSet2[] IP)
	* @see ubisec.net.IPAddress#IPAddress(String IP)
	* @see ubisec.net.IPAddress#IPAddress(InetAddress IP)
	* 
	*/
    public IPAddress()
    {
	address[0] = new BitSet2();
	address[1] = new BitSet2(); 
	address[2] = new BitSet2();
	address[3] = new BitSet2();
    }

    /**
	*
	* Creates an IPAddress object based on the given string IP address.
	*  
	* @see ubisec.net.IPAddress#IPAddress()
	* @see ubisec.net.IPAddress#IPAddress(BitSet2[] IP)
	* @see ubisec.net.IPAddress#IPAddress(InetAddress IP)
	* 
	*/
    public IPAddress(String IP)
    {
	setIPAddress(IP);
    }

    /**
	*
	* Creates an IPAddress object based on the given InetAddress object.
	* 
	* @see ubisec.net.IPAddress#IPAddress() 
	* @see ubisec.net.IPAddress#IPAddress(String IP)
	* @see ubisec.net.IPAddress#IPAddress(BitSet2[] IP)
	* 
	*/
    public IPAddress(InetAddress IP)
    {
	setIPAddress(IP);
    }

    /**
	*
	* Creates an IPAddress object and set the IP address value to the given BitSet2 array.
	*  
	* @see ubisec.net.IPAddress#IPAddress()
	* @see ubisec.net.IPAddress#IPAddress(String IP)
	* @see ubisec.net.IPAddress#IPAddress(InetAddress IP)
	* 
	*/
    public IPAddress(BitSet2[] IP)
    {
	address = IP;
    }

    /**
	*
	* Sets IP address of this IPAddress object to the IP address presented by the given string.
	*  
	* @see ubisec.net.IPAddress#setIPAddress(BitSet2[] IP)
	* @see ubisec.net.IPAddress#setIPAddress(InetAddress IP)
	* 
	*/
    public void setIPAddress(String IP)
    {
	int beginIndex=0;
	int endIndex=IP.indexOf(".");
	String tmp = IP.substring(beginIndex,endIndex);
	Integer inttmp = new Integer(tmp);
	address[0] = new BitSet2(inttmp.byteValue());
	
	beginIndex=endIndex+1;
	endIndex=IP.indexOf(".", beginIndex);
	tmp = IP.substring(beginIndex,endIndex);
	inttmp = new Integer(tmp); 
	address[1] = new BitSet2(inttmp.byteValue());

	beginIndex=endIndex+1;
	endIndex=IP.indexOf(".", beginIndex);
	tmp = IP.substring(beginIndex,endIndex);
	inttmp = new Integer(tmp);
	address[2] = new BitSet2(inttmp.byteValue());
	
	beginIndex=endIndex+1;
	tmp = IP.substring(beginIndex);
	inttmp = new Integer(tmp);
	address[3] = new BitSet2(inttmp.byteValue());
    }

    /**
	*
	* Sets IP address of this IPAddress object to the IP address contained in the given InetAddress object.
	*  
	* @see ubisec.net.IPAddress#setIPAddress(String IP)
	* @see ubisec.net.IPAddress#setIPAddress(BitSet2[] IP)
	* 
	*/
    public void setIPAddress(InetAddress IP)
    {
	setIPAddress(IP.getHostAddress()); 
    }

    /**
	*
	* Sets the array of BitSet2 of this IPAddress object to the given array of BitSet2.
	*  
	* @see ubisec.net.IPAddress#setIPAddress(String IP)
	* @see ubisec.net.IPAddress#setIPAddress(InetAddress IP)
	* 
	*/
    public void setIPAddress(BitSet2[] IP)
    {
	address = IP;
    }
    
    /**
	*
	* Returns the set of bits that contains the IP address of this IPAddress object.
	*  
	* @return the raw IP address in an array of BitSet2.
	* @see ubisec.net.IPAddress#getHostIPAddress()
	* @see ubisec.net.IPAddress#getIPInetAddress()
	* 
	*/
    public BitSet2[] getIPAddress()
    {
	return address;
    }

    
    /**
	*
	* Returns the IP address string in textual presentation.
	*  
	* @return the raw IP address in a string format.
	* @see ubisec.net.IPAddress#getIPAddress()
	* @see ubisec.net.IPAddress#getIPInetAddress()
	* 
	*/
    public String getHostIPAddress()
    {
	return (address[0].getInt())+"."+(address[1].getInt())+"."+(address[2].getInt())+"."+(address[3].getInt());
    }

    /**
	*
	* Returns the IP address into an InetAddress object.
	*  
	* @return an InetAddress object of this IP address.
	* @see ubisec.net.IPAddress#getIPAddress()
	* @see ubisec.net.IPAddress#getHostIPAddress()
	* 
	*/
    public InetAddress getIPInetAddress()
    {
	try
	    {
		return InetAddress.getByName(getHostIPAddress());
	    }
	catch (Exception ex)
	    {
		System.out.println("Error at IPAddress.getIPInetAddress()");
	    }
	return null;
    }

    /**
	*
	* Compares the IP address of this IPAddress object against the specified string IP address . The result is true if and only if the argument is not null and it represents the same IP address as this object. 
    * <p>
    * 
    * Two instances of IPAddress represent the same IP address if the string returned by
    * getHostIPAddress is the same for both. 
	*  
	* @return true if the IP addresses are the same; false otherwise.
	* @param IP address in string format 
	* @see ubisec.net.IPAddress#equals(IPAddress IP)
	* 
	*/
    private boolean equals(String IP)
    {
	if (IP.equals(getHostIPAddress())) return true;
	return false;
    }

    /**
	*
	* Compares the IP address of this IPAddress object against the IP address contained in the specified InetAddress object. The result is true if and only if the argument is not null and it represents the same IP address as this object. 
    * <p>
    * 
    * Two instances of IPAddress represent the same IP address if the string returned by
    * getHostIPAddress is the same for both. 
	*  
	* @return true if the IP addresses are the same; false otherwise.
	* @param an InetAddress object. 
	* @see ubisec.net.IPAddress#equals(String IP)
	* 
	*/
    private boolean equals(IPAddress IP)
    {
	if (IP.getHostIPAddress().equals(getHostIPAddress())) return true;
	return false;
    }
    
    /**
	*
	* Compares the IP address of this IPAddress object against the IP address contained in the specified object. The result is true if and only if the argument is not null and it represents the same IP address as this object. 
    * <p>
    * Two instances of IPAddress represent the same IP address if the string returned by
    * getHostIPAddress is the same for both. 
	*  
	* @return true if the objects are the same or have got a same IP address; false otherwise.
	* @param the object to compare against.  
	*  
	*/
    public boolean equals(Object obj)
    {
    if (obj.getClass().getName().equals("java.lang.String")) return equals((String)obj);
    if (obj.getClass().getName().equals("java.net.InetAddress")) return equals((InetAddress)obj);
    if (obj.getClass().getName().equals("ubisec.net.IPAddress")) return equals(((IPAddress)obj).getHostIPAddress());
    return false;
    }
}
