package ubisec.net;


import ubisec.configuration.Setting;
import java.util.Vector;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;


/**
*
* This class provides some network coverage information of the node where it's running. 
* <p>
* Mainly, this class provides the number and the IP address of 1) all the neighbors, 2) the neighbors which are elected as directories, and 3) the others, over one hop. 
* In addition, this class periodicaly broadcasts a simple message over the network on a specific port, the defaut value of the message is "1" if the node is a directory, or "0" otherwise.  
* When such message is received from another node, the IP address of this node is extracted and added to the corresponding list or just corresponding TTL is updated when the IP address already exist. 
* The neighbors lists are periodically cheked and updated, if some neighbors don't send any messages after the TTL time is spent are automaticaly removed from the neighbors list. 
* <p>
* If some nodes are contained in the network vicinity and are not executing a NetworkCover instance are not added to the neighbors list.    
* 
* @author Rafik CHIBOUT 
* @author Francoise SAILHAN
* @version 0.1 sept. 2004
*
* 
* 
*/

public class NetworkCover
{
	private static int NEIGHBORS_ADVERTISEMENT=0;
	private static int NEIGHBORS_LISTS_TTL_UPDATE=1;
	private static int NEIGHBORS_ADVERTISEMENT_VALUE=2000;
	private static int NEIGHBORS_LISTS_TTL_UPDATE_VALUE=1000;
	private static int HOPS_NUMBER_VALUE=1;
	private int initTTL = 20000;
	
	private Vector MPR = new Vector();
		
	private int neighborsPort =0;
    private String messageType = "0";  // 0:SD case, 1: Directory case 
    private String[][] neighborsDirectories = new String[100][500];
    private int[] neighborsDirectoriesLevel = new int[100];
    
    private int[] TTL4Dir = new int[100];
    private int cptDirectory = 0;
    private String[][] neighborsSD = new String[100][500];
    private int[] neighborsSDLevel = new int[100];
    
    private int[] TTL4SD  = new int[100];
    private int cptSD = 0;
    private String[][] neighborsNotCoveredSD = new String[100][500];
    private int[] neighborsNotCoveredSDLevel = new int[100];
    
    private int[] TTL4NotCoveredSD = new int[100];
    private int cptNotCoveredSD=0;
    
    private boolean start = true;
    private NetworkManager manager = null;
    
    private int currentLevel = 1; // used to broadcast
    private int MPRLevel=0; // in ordre to know the level of calculated MPR
    
    private static Logger logger = Logger.getLogger(NetworkCover.class); 
    private Level loggerLevel = Level.INFO;
    
    public void setProperty(int propertyName, int propertyValue)
    {
    	switch (propertyName)
	    {
	    	case 0: NEIGHBORS_ADVERTISEMENT_VALUE=propertyValue;break;
	    	case 1: NEIGHBORS_LISTS_TTL_UPDATE_VALUE=propertyValue;break;
	    	case 2: HOPS_NUMBER_VALUE=propertyValue;break;
	    }
    }
    
    public int getProperty(int propertyName)
    {
    	switch (propertyName)
	    {
	    	case 0: return NEIGHBORS_ADVERTISEMENT_VALUE;
	    	case 1: return NEIGHBORS_LISTS_TTL_UPDATE_VALUE;
	    	case 2: return HOPS_NUMBER_VALUE;
	    	default : return -1;
	    }
    }
    
    /**
     * creates and starts a NetworkCover object which sends the messages and listens on the specified port. 
     * @param port the port number.
     * 
     */
    public NetworkCover(int port)
    {
    	    	start(port);
    }
    
    /**
     * 
     * creates and starts a NetworkCover object which sends periodically the given message and listens on the specified port.
     * @param port the port number
     * @param message the message to send
     * 
     */
    public NetworkCover(int port ,String message)
    {
    	this.messageType = message;
    	start(port);
    }
    
    /**
     * 
     * Starts the threads: neighbors deamon listener, the message sender and the TTLUpdate.
     * @param port the port number.
     * 
     */
    
    public void start(int port)
    {
    	
    	SimpleLayout layout = new SimpleLayout();
    	ConsoleAppender appender = new ConsoleAppender(layout,ConsoleAppender.SYSTEM_OUT);
    	appender.setName("screen");
    	appender.activateOptions();
		logger.addAppender(appender);
    	logger.setLevel(loggerLevel);
    	
		manager = new NetworkManager();
		manager.listenToPort(port);
		    	
       	NEIGHBORS_ADVERTISEMENT_VALUE=Setting.COVER_NEIGHBORS_ADVERTISEMENT_VALUE;
    	NEIGHBORS_LISTS_TTL_UPDATE_VALUE=Setting.COVER_NEIGHBORS_LISTS_TTL_UPDATE_VALUE;
    	HOPS_NUMBER_VALUE=Setting.HopsNumber;
    	//HOPS_NUMBER_VALUE=3;
    	neighborsPort=port;
    	start = true;
    	Thread deamon = NeighborsDeamon();
    	Thread sender = NeighborsSender();
    	Thread ttlupdate = TTLUpdate();
    	sender.start();
    	deamon.start();
    	ttlupdate.start();
    	updateMPRlist(1);
    	try
	    	{
    		//Thread.sleep(10000);
	    	}
    	catch(Exception ex)
	    	{
    			System.out.println("Error at NetworkCover.start:sleep");
	    	}
    		//updateMPRlist(1);
    }

    /**
     * Stops the threads: neighbors deamon listener, the message sender and the TTLUpdate.
     * 
     */
    public void stop()
    {
    	 start = false;
    }
    
    /**
     * Sets the message which will be to send to the other nodes.
     * @param message the string message.
     * 
     */
    public void setMessageType(String message)
    {
    	this.messageType = message;
    }
    
    private boolean isContains(String[] vect , String val)
    {
       	int i=0;
    	while(i<vect.length)
    	{
    		if (vect[i]!=null && vect[i].equals(val)) return true;
    		i++;
    	}
    	return false;
    }
    
    private int indexOf(String[] vect , String val)
    {
    	int i=0;
    	while(i<vect.length)
    	{
    		if (vect[i]!=null && vect[i].equals(val)) return i;
    		i++;
    	}
    	return -1;
    }
    
    private int indexOf(String[] vect , int beginIndex, int length, String val)
    {
    	int i=beginIndex;
    	while(i<beginIndex+length && i<vect.length)
    	{
    		if (vect[i]!=null && vect[i].equals(val)) return i;
    		i++;
    	}
    	return -1;
    }
    
    private void addTo(String[] vect , int beginIndex , int length , String val)
    {
    	//if (length+beginIndex>vect.length) return;
    	int i=beginIndex;
    	while (i<length+beginIndex && i<vect.length)
    	{
    		if (vect[i]!=null && vect[i]== null || vect[i].equals(val)) 
    		{
    			vect[i]=val;
    			return;
    		}
    		i++;
    	}
    }
    
    private void addTo(String[] vect, String val)
    {
    	int i=0;
    	while (i<vect.length)
    	{
    		if (vect[i]== null || vect[i].equals(val)) 
    		{
    			vect[i]=val;
    			return;
    		}
    		i++;
    	}
    }
    
    /**
     * 
     * @param vect
     * @param val
     * 
     */
    private void removeFrom(String[] vect, String val)
    {
    	int index = indexOf(vect, val);
    	if (index!=-1) vect[index]=""; 
    }
    
    private void removeFrom(String[] vect, int beginIndex, int length, String val)
    {
    	int index = indexOf(vect, val);
    	if (index!=-1 && index>beginIndex && index<beginIndex+length)
    		vect[index]="";
    }
    
    private boolean isContains(String[][] vect, int lines, int columns, String val)
    {
    	int i=0;
    	while(i<lines)
    	{
    		if (isContains(vect[i], val)) return true;
    		i++;
    	}
    	return false;
    }
    
    private void addTo(String[][] vect, int line, int beginIndex , int length, String val)
    {
    	addTo(vect[line], beginIndex, length, val);
    }
    
    private void addSDNeighbor(String ip, int counter, int sd, int notCoveredSD, int dir, String[] ipSDNeighbors, String[] ipNotCoveredSDNeighbors, String[] ipDirNeighbors)
    {
    	//if (dir == 0) 
    		//{
    		//	addNotCoveredSDNeighbor(ip, counter, sd, notCoveredSD, dir, ipSDNeighbors, ipNotCoveredSDNeighbors, ipDirNeighbors);
    		//	return;
    		//}
    
    boolean mustUpdate = false;
    int index =0;
	while (index <100 && (neighborsSD[index][0] == null || !neighborsSD[index][0].equals(ip))) index++;
	if (index<100) 
	{ 
		if (neighborsSDLevel[index]<counter+1) return;
		if (neighborsSDLevel[index]>counter+1) mustUpdate = true;
		TTL4SD[index]= initTTL;
		// look if the neighbors of this neighbor have been changed
		int f=1;
		while (!mustUpdate && f<500)
		{
			if (neighborsSD[index][f]!=null && !neighborsSD[index][f].equals(""))
			{
				int g=0;
				while (g<ipSDNeighbors.length && !neighborsSD[index][f].equals(ipSDNeighbors[g]))
					g++;
				if (g>=ipSDNeighbors.length) 
					{
					g=0;
					while (g<ipNotCoveredSDNeighbors.length && !neighborsSD[index][f].equals(ipNotCoveredSDNeighbors[g]))
						g++;
					if (g>=ipNotCoveredSDNeighbors.length) 
					{
						g=0;
						while (g<ipDirNeighbors.length && !neighborsSD[index][f].equals(ipDirNeighbors[g]))
							g++;
						if (g>=ipDirNeighbors.length) 
						{
							//System.out.println("the neighbors "+neighborsSD[index][f]+" is not found into the new neighbors of "+ip);
							mustUpdate= true;
						}
					}
					}
			}
			f++;
		}
		
		int h=1;
		f=0;
		while (!mustUpdate && f<ipSDNeighbors.length)
		{
			while (h<500 && (neighborsSD[index][h]==null || !neighborsSD[index][h].equals(ipSDNeighbors[f])))
				h++;
			if (h>=500) 
				{
				//System.out.println("the new SD neighbor "+ipSDNeighbors[f]+" is not found into the neighbors of "+ip);
				mustUpdate = true;
				}
			f++;
		}
		
		f=0;
		while (!mustUpdate && f<ipNotCoveredSDNeighbors.length)
		{
			h=1;
			while (h<500 && (neighborsSD[index][h]==null || !neighborsSD[index][h].equals(ipNotCoveredSDNeighbors[f])))
				h++;
			if (h>=500) 
				{
				//System.out.println("the new NotCoveredSD neighbor "+ipNotCoveredSDNeighbors[f]+" is not found into the neighbors of "+ip);
				mustUpdate = true;
				}
			f++;
		}
		
		f=0;
		while (!mustUpdate && f<ipDirNeighbors.length)
		{
			h=1;
			while (h<500 && (neighborsSD[index][h]==null || !neighborsSD[index][h].equals(ipDirNeighbors[f])))
				h++;
			if (h>=500) 
				{
				//System.out.println("the new Dir neighbor "+ipDirNeighbors[f]+" is not found into the neighbors of "+ip);
				mustUpdate = true;
				}
			f++;
		}
		// end of change verification
		
		for (int i=1; i<=200; i++)
	    	neighborsSD[index][i]="";
		for (int i=0;i<ipSDNeighbors.length;i++)
			neighborsSD[index][i+1]=ipSDNeighbors[i];
				
		for (int i=201; i<=400; i++)
	    	neighborsSD[index][i]="";
		for (int i=0;i<ipNotCoveredSDNeighbors.length;i++)
			neighborsSD[index][i+201]=ipNotCoveredSDNeighbors[i];
				
		for (int i=401; i<500; i++)
	    	neighborsSD[index][i]="";
		for (int i=0;i<ipDirNeighbors.length;i++)
			neighborsSD[index][i+401]=ipDirNeighbors[i];
		
		// remove spent calcul time from TTL
		TTL4SD[index]= initTTL;
		neighborsSDLevel[index]=counter+1;
	}
	else 
	    {
		index=0;
		while (index < 100 && (neighborsDirectories[index][0]==null || !neighborsDirectories[index][0].equals(ip))) index++;
		if (index < 100) 
		    {
			for (int i=0;i<500;i++)
				neighborsDirectories[index][i]="";
				neighborsDirectoriesLevel[index]=-1;
			TTL4Dir[index]=0;
			cptDirectory--;
		    }
		index=0;
		while (index <100 && (neighborsNotCoveredSD[index][0]==null || !neighborsNotCoveredSD[index][0].equals(ip))) index++;
		if (index <100) 
		    {
			for (int i=0;i<100;i++)
				neighborsNotCoveredSD[index][i]="";
			neighborsNotCoveredSDLevel[index]=-1;
			TTL4NotCoveredSD[index]=0;
			cptNotCoveredSD--;
		    }
		index =0;
		while (index < 100 && neighborsSD[index][0]!=null && !neighborsSD[index][0].equals("")) index++;
		if (index < 100)
		    {
			logger.debug("****SD neighbor is added ip="+ip+" level="+(counter+1));
			neighborsSDLevel[index]=counter+1;
			neighborsSD[index][0]=ip;
			
			TTL4SD[index] = initTTL;
			for (int i=1; i<=200; i++)
		    	neighborsSD[index][i]="";
			for (int i=0;i<ipSDNeighbors.length;i++)
				neighborsSD[index][i+1]=ipSDNeighbors[i];
			
			for (int i=201; i<=400; i++)
		    	neighborsSD[index][i]="";
			for (int i=0;i<ipNotCoveredSDNeighbors.length;i++)
				neighborsSD[index][i+201]=ipNotCoveredSDNeighbors[i];
					
			for (int i=401; i<500; i++)
		    	neighborsSD[index][i]="";
			for (int i=0;i<ipDirNeighbors.length;i++)
				neighborsSD[index][i+401]=ipDirNeighbors[i];
						
			
			cptSD++;
			mustUpdate = true;
		    }
	    }
	
	if (mustUpdate) updateMPRlist(counter+1);
    }

    /**
     * 
     * @param ip
     * @param counter
     * @param sd
     * @param notCoveredSD
     * @param dir
     * @param ipSDNeighbors
     * @param ipNotCoveredSDNeighbors
     * @param ipDirNeighbors
     * 
     */
    private void addNotCoveredSDNeighbor(String ip, int counter, int sd, int notCoveredSD, int dir, String[] ipSDNeighbors, String[] ipNotCoveredSDNeighbors, String[] ipDirNeighbors)
    {
    //SD not covered by directory
    boolean mustUpdate = false;
    int index =0;
	while (index <100 && (neighborsNotCoveredSD[index][0] == null || !neighborsNotCoveredSD[index][0].equals(ip))) index++;
	if (index<100) 
	{ 
		if (neighborsNotCoveredSDLevel[index]<counter+1) return;
		if (neighborsNotCoveredSDLevel[index]>counter+1) mustUpdate = true;
		TTL4NotCoveredSD[index]= initTTL;
		// look if the neighbors of this neighbor have been changed
		int f=1;
		while (!mustUpdate && f<500)
		{
			if (neighborsNotCoveredSD[index][f]!=null && !neighborsNotCoveredSD[index][f].equals(""))
			{
				int g=0;
				while (g<ipSDNeighbors.length && !neighborsNotCoveredSD[index][f].equals(ipSDNeighbors[g]))
					g++;
				if (g>=ipSDNeighbors.length) 
				{
					g=0;
					while (g<ipNotCoveredSDNeighbors.length && !neighborsNotCoveredSD[index][f].equals(ipNotCoveredSDNeighbors[g]))
						g++;
					if (g>=ipNotCoveredSDNeighbors.length) 
					{
						g=0;
						while (g<ipDirNeighbors.length && !neighborsNotCoveredSD[index][f].equals(ipDirNeighbors[g]))
							g++;
						if (g>=ipDirNeighbors.length) mustUpdate= true;
					}
				}
			}
			f++;
		}
		
		int h=1;
		f=0;
		while (!mustUpdate && f<ipSDNeighbors.length)
		{
			while (h<500 && (neighborsNotCoveredSD[index][h]==null || !neighborsNotCoveredSD[index][h].equals(ipSDNeighbors[f])))
				h++;
			if (h>=500) mustUpdate = true;
			f++;
		}
		
		f=0;
		while (!mustUpdate && f<ipNotCoveredSDNeighbors.length)
		{
			h=1;
			while (h<500 && (neighborsNotCoveredSD[index][h]==null || !neighborsNotCoveredSD[index][h].equals(ipNotCoveredSDNeighbors[f])))
				h++;
			if (h>=500) mustUpdate = true;
			f++;
		}
		
		f=0;
		while (!mustUpdate && f<ipDirNeighbors.length)
		{
			h=1;
			while (h<500 && (neighborsNotCoveredSD[index][h]==null || !neighborsNotCoveredSD[index][h].equals(ipDirNeighbors[f])))
				h++;
			if (h>=500) mustUpdate = true;
			f++;
		}
		// end of change verification
		
		for (int i=1; i<=200; i++)
	    	neighborsNotCoveredSD[index][i]="";
		for (int i=0;i<ipSDNeighbors.length;i++)
			neighborsNotCoveredSD[index][i+1]=ipSDNeighbors[i];
			
		for (int i=201; i<=400; i++)
	    	neighborsNotCoveredSD[index][i]="";
		for (int i=0;i<ipNotCoveredSDNeighbors.length;i++)
			neighborsNotCoveredSD[index][i+201]=ipNotCoveredSDNeighbors[i];
			
		for (int i=401; i<500; i++)
	    	neighborsNotCoveredSD[index][i]="";
		for (int i=0;i<ipDirNeighbors.length;i++)
			neighborsNotCoveredSD[index][i+401]=ipDirNeighbors[i];
		
		// remove spent calcul time from TTL
		TTL4NotCoveredSD[index]= initTTL;
		neighborsNotCoveredSDLevel[index]=counter+1;
	}
	else 
	    {
		index=0;
		while (index < 100 && (neighborsDirectories[index][0]==null || !neighborsDirectories[index][0].equals(ip))) index++;
		if (index < 100) 
		    {
			for (int i=0;i<500;i++)
				neighborsDirectories[index][i]="";
			neighborsDirectoriesLevel[index]=-1;
			TTL4Dir[index]=0;
			cptDirectory--;
		    }
		index =0;
		while (index <100 && (neighborsSD[index][0]==null || !neighborsSD[index][0].equals(ip))) index++;
		if (index <100) 
		    {
			for (int i=0;i<500;i++)
			neighborsSD[index][i]="";
			neighborsSDLevel[index]=-1;
			TTL4SD[index]=0;
			cptSD--;
		    }
		index =0;
		while (index < 100 && neighborsNotCoveredSD[index][0]!=null && !neighborsNotCoveredSD[index][0].equals("")) index++;
		if (index < 100)
		    {
			logger.debug("****not covered SD neighbor is added ip="+ip+" level="+(counter+1));
			neighborsNotCoveredSDLevel[index]=counter+1;
			neighborsNotCoveredSD[index][0]=ip;
			TTL4NotCoveredSD[index] = initTTL;
			
			for (int i=1; i<=200; i++)
		    	neighborsNotCoveredSD[index][i]="";
			for (int i=0;i<ipSDNeighbors.length;i++)
				neighborsNotCoveredSD[index][i+1]=ipSDNeighbors[i];
						
			for (int i=201; i<=400; i++)
		    	neighborsNotCoveredSD[index][i]="";
			for (int i=0;i<ipNotCoveredSDNeighbors.length;i++)
				neighborsNotCoveredSD[index][i+201]=ipNotCoveredSDNeighbors[i];
						
			for (int i=401; i<500; i++)
		    	neighborsNotCoveredSD[index][i]="";
			for (int i=0;i<ipDirNeighbors.length;i++)
				neighborsNotCoveredSD[index][i+401]=ipDirNeighbors[i];
			
			cptNotCoveredSD++;
			mustUpdate = true;
		    }
	    }
	if (mustUpdate) updateMPRlist(counter+1);
    }
    
    /**
     * 
     * @param ip
     * @param counter
     * @param sd
     * @param notCoveredSD
     * @param dir
     * @param ipSDNeighbors
     * @param ipNotCoveredSDNeighbors
     * @param ipDirNeighbors
     * 
     */
    private void addDirectoryNeighbor(String ip, int counter, int sd, int notCoveredSD, int dir, String[] ipSDNeighbors, String[] ipNotCoveredSDNeighbors, String[] ipDirNeighbors)
    {
    boolean mustUpdate = false;
    int index =0;
    while (index <100 && (neighborsDirectories[index][0]==null || !neighborsDirectories[index][0].equals(ip))) index++;
	if (index<100)
	{
		if (neighborsDirectoriesLevel[index]<counter+1) return;
		if (neighborsDirectoriesLevel[index]>counter+1) mustUpdate = true;
		TTL4Dir[index]= initTTL;
		// look if the neighbors of this neighbor have been changed
		int f=1;
		while (!mustUpdate && f<500)
		{
			if (neighborsDirectories[index][f]!=null && !neighborsDirectories[index][f].equals(""))
			{
				int g=0;
				while (g<ipSDNeighbors.length && !neighborsDirectories[index][f].equals(ipSDNeighbors[g]))
					g++;
				if (g>=ipSDNeighbors.length) 
					{
					g=0;
					while (g<ipNotCoveredSDNeighbors.length && !neighborsDirectories[index][f].equals(ipNotCoveredSDNeighbors[g]))
						g++;
					if (g>=ipNotCoveredSDNeighbors.length) 
					{
						g=0;
						while (g<ipDirNeighbors.length && !neighborsDirectories[index][f].equals(ipDirNeighbors[g]))
							g++;
						if (g>=ipDirNeighbors.length) 
							{
							mustUpdate= true;
							//System.out.println("the neighbors "+neighborsDirectories[index][f]+" is not found into the new neighbors of "+ip);
							}
					}
					}
			}
			f++;
		}
		int h=1;
		f=0;
		while (!mustUpdate && f<ipSDNeighbors.length)
		{
			while (h<500 && (neighborsDirectories[index][h]==null || !neighborsDirectories[index][h].equals(ipSDNeighbors[f])))
				h++;
			if (h>=500) 
				{
				//System.out.println("the new SD neighbor "+ipSDNeighbors[f]+" is not found into the neighbors of "+ip);
				mustUpdate = true;
				}
			f++;
		}
		
		f=0;
		while (!mustUpdate && f<ipNotCoveredSDNeighbors.length)
		{
			h=1;
			while (h<500 && (neighborsDirectories[index][h]==null || !neighborsDirectories[index][h].equals(ipNotCoveredSDNeighbors[f])))
				h++;
			if (h>=500)
				{
				//System.out.println("the new NotCoveredSD neighbor "+ipNotCoveredSDNeighbors[f]+" is not found into the neighbors of "+ip);
				mustUpdate = true;
				}
			f++;
		}
		
		f=0;
		while (!mustUpdate && f<ipDirNeighbors.length)
		{
			h=1;
			while (h<500 && (neighborsDirectories[index][h]==null || !neighborsDirectories[index][h].equals(ipDirNeighbors[f])))
				h++;
			if (h>=500) 
				{
				//System.out.println("the new Dir neighbor "+ipDirNeighbors[f]+" is not found into the neighbors of "+ip);
				mustUpdate = true;
				}
			f++;
		}
	
		// end of change verification
		for (int i=1; i<=200; i++)
		    neighborsDirectories[index][i]="";
		for (int i=0;i<ipSDNeighbors.length;i++)
			neighborsDirectories[index][i+1]=ipSDNeighbors[i];
		
		for (int i=201; i<=400; i++)
	    	neighborsDirectories[index][i]="";
		for (int i=0;i<ipNotCoveredSDNeighbors.length;i++)
			neighborsDirectories[index][i+201]=ipNotCoveredSDNeighbors[i];
		
		for (int i=401; i<500; i++)
		    neighborsDirectories[index][i]="";
		for (int i=0;i<ipDirNeighbors.length;i++)
			neighborsDirectories[index][i+401]=ipDirNeighbors[i];
		// remove spent calcul time from TTL
		TTL4Dir[index]= initTTL;
		neighborsDirectoriesLevel[index]=counter+1;
	}
	else 
	    {
		index=0;
		while (index <100 && (neighborsSD[index][0]==null || !neighborsSD[index][0].equals(ip))) index++;
		if (index <100) 
		    {
			for (int i=0;i<500;i++)
				neighborsSD[index][i]="";
				neighborsSDLevel[index]=-1;
			TTL4SD[index]=0;
			cptSD--;
		    }
		index=0;
		while (index <100 && (neighborsNotCoveredSD[index][0]==null || !neighborsNotCoveredSD[index][0].equals(ip))) index++;
		if (index <100) 
		    {
			for (int i=0;i<500;i++)
				neighborsNotCoveredSD[index][i]="";
			neighborsNotCoveredSDLevel[index]=counter+1;
			TTL4NotCoveredSD[index]=0;
			cptNotCoveredSD--;
		    }
		index =0;
		while (index <100 && neighborsDirectories[index][0]!=null && !neighborsDirectories[index][0].equals("")) index++;
		if (index < 100)
		    {
			neighborsDirectoriesLevel[index]=counter+1;
			neighborsDirectories[index][0]=ip;
			TTL4Dir[index] = initTTL; 
			logger.debug("Dir neighbors is add ip="+ip+" level="+(counter+1));
			for (int i=1; i<=200; i++)
		    	neighborsDirectories[index][i]="";
			for (int i=0;i<ipSDNeighbors.length;i++)
				neighborsDirectories[index][i+1]=ipSDNeighbors[i];
			
			for (int i=201; i<=400; i++)
		    	neighborsDirectories[index][i]="";
			for (int i=0;i<ipNotCoveredSDNeighbors.length;i++)
				neighborsDirectories[index][i+201]=ipNotCoveredSDNeighbors[i];
			
			for (int i=401; i<500; i++)
			    neighborsDirectories[index][i]="";
			for (int i=0;i<ipDirNeighbors.length;i++)
				neighborsDirectories[index][i+401]=ipDirNeighbors[i];
				
			
			cptDirectory++;
			mustUpdate = true;
		    }
	    }
	if (mustUpdate) updateMPRlist(counter+1);
    }

    /**
     * Returns the number of the neighbors in the network vicinity over one hop, which are performing an instance of NetworkCover. It gives the number of the directories in the vicinity added to the number of the others nodes.
     * @return the number of neighbors.
     */
    public int getNeighborsNumber()
    {
    	return cptSD+cptNotCoveredSD+cptDirectory;
    }

    /**
     * Returns the number of the neighbors in the network vicinity over one hop, which are performing an instance of NetworkCover and are not dicrectories.
     * @return the number of the neighbors that are not directories.
     */
    public int getSDNeighborsNumber()
    {
    	return cptSD;
    }
    /**
     * 
     * @return
     */
    public int getNotCoveredSDNeighborsNumber()
    {
    	// return cptNotCoveredSD + myself if I'am not covered
    	return cptNotCoveredSD;
    }

    /**
     * Returns the number of the directories neighbors in the network vicinity over one hop.
     * @return the number of directories neighbors.
     * 
     */
    public int getDirectoriesNeighborsNumber()
    {
    	return cptDirectory;
    }

    public int getSDNeighborsOfDirectoriesNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=1; column<201;column++)
    			if (neighborsDirectories[line][column]!=null &&
        			!neighborsDirectories[line][column].equals("") &&
					!isContains(tmp,neighborsDirectories[line][column].substring(0, neighborsDirectories[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsDirectories[line][column].substring(0, neighborsDirectories[line][column].indexOf(":")));
    				forReturn++;
    			}
    	return forReturn;
    }
    
    public int getNotCoveredSDNeighborsOfDirectoriesNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=201; column<401;column++)
    			if (neighborsDirectories[line][column]!=null &&
        			!neighborsDirectories[line][column].equals("") &&
					!isContains(tmp,neighborsDirectories[line][column].substring(0, neighborsDirectories[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsDirectories[line][column].substring(0, neighborsDirectories[line][column].indexOf(":")));
    				forReturn++;
    			}
    	
    	return forReturn;
    }
    
    public int getDirNeighborsOfDirectoriesNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=401; column<500;column++)
    			if (neighborsDirectories[line][column]!=null &&
        			!neighborsDirectories[line][column].equals("") &&
					!isContains(tmp,neighborsDirectories[line][column].substring(0, neighborsDirectories[line][column].indexOf(":")))) 
    				{
    				addTo(tmp,neighborsDirectories[line][column].substring(0, neighborsDirectories[line][column].indexOf(":")));
    				forReturn++;
    				}
    	return forReturn;
    }
    
    public int getSDNeighborsOfSDNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=1; column<201;column++)
    			if (neighborsSD[line][column]!=null && 
        			!neighborsSD[line][column].equals("") && 
					!isContains(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")))) 
    				{
    				addTo(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")));
    				forReturn++;
    				}
    	return forReturn;
    }
    
    public int getNotCoveredSDNeighborsOfSDNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=201; column<401;column++)
    			if (neighborsSD[line][column]!=null && 
        			!neighborsSD[line][column].equals("") && 
					!isContains(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")))) 
    				{
    				addTo(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")));
    				forReturn++;
    				}
    	return forReturn;
    }
    
    public int getDirNeighborsOfSDNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=401; column<500;column++)
    			if (neighborsSD[line][column]!=null && 
        			!neighborsSD[line][column].equals("") && 
					!isContains(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    	
    	return forReturn;
    }
    
    // not covered SD
    public int getSDNeighborsOfNotCoveredSDNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=1; column<201;column++)
    			if (neighborsNotCoveredSD[line][column]!=null && 
        			!neighborsNotCoveredSD[line][column].equals("") &&
					!isContains(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    	
    	return forReturn;
    }
    
    public int getNotCoveredSDNeighborsOfNotCoveredSDNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=201; column<401;column++)
    			if (neighborsNotCoveredSD[line][column]!=null && 
        			!neighborsNotCoveredSD[line][column].equals("") &&
					!isContains(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    	
    	return forReturn;
    }
    
    public int getDirNeighborsOfNotCoveredSDNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[500];
    	for (int line=0; line<100; line++)
    		for (int column=401; column<500;column++)
    			if (neighborsNotCoveredSD[line][column]!=null && 
        			!neighborsNotCoveredSD[line][column].equals("") &&
    				!isContains(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    	return forReturn;
    }
        
    public int getSDNeighborsOfNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[1000];
    	for (int line=0; line<100; line++)
    		for (int column=1; column<201;column++)
    		{
    			if (neighborsNotCoveredSD[line][column]!=null && 
    				!neighborsNotCoveredSD[line][column].equals("") &&
    				!isContains(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    			
    			if (neighborsSD[line][column]!=null && 
    				!neighborsSD[line][column].equals("") && 
					!isContains(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    			if (neighborsDirectories[line][column]!=null &&
    				!neighborsDirectories[line][column].equals("") &&
					!isContains(tmp,neighborsDirectories[line][column].substring(0,neighborsDirectories[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsDirectories[line][column].substring(0,neighborsDirectories[line][column].indexOf(":")));
    				forReturn++;
    			}
    		}
    	return forReturn;
    	//return getSDNeighborsOfSDNeighborsNumber()+getSDNeighborsOfNotCoveredSDNeighborsNumber()+getSDNeighborsOfDirectoriesNeighborsNumber();
    }
    
        
    public int getNotCoveredSDNeighborsOfNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[1000];
    	for (int line=0; line<100; line++)
    		for (int column=201; column<401;column++)
    		{
    			if (neighborsNotCoveredSD[line][column]!=null && 
        			!neighborsNotCoveredSD[line][column].equals("") &&
					!isContains(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    			if (neighborsSD[line][column]!=null && 
        			!neighborsSD[line][column].equals("") && 
					!isContains(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    			if (neighborsDirectories[line][column]!=null &&
        			!neighborsDirectories[line][column].equals("") &&
					!isContains(tmp,neighborsDirectories[line][column].substring(0,neighborsDirectories[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsDirectories[line][column].substring(0,neighborsDirectories[line][column].indexOf(":")));
    				forReturn++;
    			}
    		}
    	return forReturn;
    	//return getNotCoveredSDNeighborsOfSDNeighborsNumber()+getNotCoveredSDNeighborsOfNotCoveredSDNeighborsNumber()+getNotCoveredSDNeighborsOfDirectoriesNeighborsNumber();
    }
    
    public int getDirNeighborsOfNeighborsNumber()
    {
    	int forReturn =0;
    	String[] tmp = new String[1000];
    	for (int line=0; line<100; line++)
    		for (int column=401; column<500;column++)
    		{
    			if (neighborsNotCoveredSD[line][column]!=null && 
        			!neighborsNotCoveredSD[line][column].equals("") &&
					!isContains(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsNotCoveredSD[line][column].substring(0,neighborsNotCoveredSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    			if (neighborsSD[line][column]!=null && 
        			!neighborsSD[line][column].equals("") && 
					!isContains(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsSD[line][column].substring(0,neighborsSD[line][column].indexOf(":")));
    				forReturn++;
    			}
    			if (neighborsDirectories[line][column]!=null &&
        			!neighborsDirectories[line][column].equals("") &&
					!isContains(tmp,neighborsDirectories[line][column].substring(0,neighborsDirectories[line][column].indexOf(":")))) 
    			{
    				addTo(tmp,neighborsDirectories[line][column].substring(0,neighborsDirectories[line][column].indexOf(":")));
    				forReturn++;
    			}
    		}
    	return forReturn;
    	//return getDirNeighborsOfSDNeighborsNumber()+getDirNeighborsOfNotCoveredSDNeighborsNumber()+getDirNeighborsOfDirectoriesNeighborsNumber();
    }
    
    /**
     * Returns the IP addresses of the neighbors in the network vicinity over one hop, which are performing an instance of NetworkCover. It gives the addresses of both the directories nodes and the others nodes.
     * @return array of string which contains the neighbors IP addresses.
     *   
     */
    public String[] getNeighborsAddresses()
    {
	String[] forReturn = new String[getNeighborsNumber()];
	int pos = 0;
	for (int index=0; index<100; index++) 
	    {
		if (neighborsDirectories[index][0]!= null && !neighborsDirectories[index][0].equals("")) 
		    {
			forReturn[pos]=neighborsDirectories[index][0];
			pos++;
		    }
		if (neighborsSD[index][0]!= null && !neighborsSD[index][0].equals("")) 
		    {
			forReturn[pos]=neighborsSD[index][0];
			pos++;
		    }
		if (neighborsNotCoveredSD[index][0]!= null && !neighborsNotCoveredSD[index][0].equals("")) 
	    	{
			forReturn[pos]=neighborsNotCoveredSD[index][0];
			pos++;
	    	}
	    }
		return forReturn;
    }
    
    /**
     * Returns the IP addresses of the directories neighbors in the network vicinity over one hop.
     * @return array of string which contains the directories neighbors IP addresses.
     *   
     */
    public String[] getDirectoriesNeighborsAddresses()
    {
    	String[] forReturn = new String[getDirectoriesNeighborsNumber()];
    	int pos = 0;
    	for (int index=0; index<100; index++) 
	    if (neighborsDirectories[index][0]!= null && !neighborsDirectories[index][0].equals("")) 
		{
		    forReturn[pos]=neighborsDirectories[index][0];
		    pos++;
		}
    	return forReturn;
    }
    
    /**
     * Returns the IP addresses of the neighbors in the network vicinity over one hop, which are performing an instance of NetworkCover and are not directories. 
     * @return array of string which contains the IP addresses of the neighbors that are not directories.
     *   
     */
    public String[] getSDNeighborsAddresses()
    {
	String[] forReturn = new String[getSDNeighborsNumber()];
	int pos = 0;
	for (int index=0; index<100; index++) 
	    if (neighborsSD[index][0]!= null && !neighborsSD[index][0].equals("")) 
		{
		    forReturn[pos]=neighborsSD[index][0];
		    pos++;
		}
	return forReturn;
    }

    /**
     * Returns the IP addresses of the neighbors in the network vicinity over one hop, which are performing an instance of NetworkCover and are not directories and are not covered by the diretories. 
     * @return array of string which contains the IP addresses of the neighbors that are not directories.
     *   
     */
    public String[] getNotCoveredSDNeighborsAddresses()
    {
    	String[] forReturn = new String[getNotCoveredSDNeighborsNumber()];
    	int pos = 0;
    	for (int index=0; index<100; index++) 
    		if (neighborsNotCoveredSD[index][0]!= null && !neighborsNotCoveredSD[index][0].equals("")) 
    		{
    			forReturn[pos]=neighborsNotCoveredSD[index][0];
    			pos++;
    		}
    	return forReturn;
    }

    public int getHighNotCoveredSDLevel()
    {
    	int forReturn = 0;
    	for (int i=0; i<100; i++)
    	{
    		if (neighborsNotCoveredSD[i][0]!=null && !neighborsNotCoveredSD[i][0].equals("") && neighborsNotCoveredSDLevel[i]>forReturn)
    			forReturn = neighborsNotCoveredSDLevel[i];
       	}
    	return forReturn;
    }
    
    private boolean isNeighbor(String ip, int level)
    {
    	for (int i=0; i<100; i++)
    		if ((neighborsDirectories[i][0]!=null && neighborsDirectories[i][0].equals(ip) && neighborsDirectoriesLevel[i]<= level) || (neighborsSD[i][0]!=null && neighborsSD[i][0].equals(ip) && neighborsSDLevel[i]<= level) || (neighborsNotCoveredSD[i][0]!=null && neighborsNotCoveredSD[i][0].equals(ip) && neighborsNotCoveredSDLevel[i]<= level)) return true;
    	return false;
    }
    
    private int levelOf(String ip)
    {
    	for (int i=0; i<100; i++)
    		if (neighborsDirectories[i][0]!=null && neighborsDirectories[i][0].equals(ip))
    			return neighborsDirectoriesLevel[i];
		    else if (neighborsSD[i][0]!=null && neighborsSD[i][0].equals(ip))
		    	return neighborsSDLevel[i];
		    else if (neighborsNotCoveredSD[i][0]!=null && neighborsNotCoveredSD[i][0].equals(ip))
		    	return neighborsNotCoveredSDLevel[i]; 
    	return -1;
    }
    
    /**
     * 
     * @param initlevel
     * 
     */
    public void updateMPRlist(int initlevel)
    {
    	//System.out.println("***********Update MPR List***********initlevel="+initlevel);
    	//System.out.println("**************MPRLevel="+MPRLevel);
    	if (initlevel==0) initlevel=1;
    	if (initlevel > MPRLevel) initlevel = MPRLevel+1;
    	
    	currentLevel=initlevel;
    	String localAddress = NetworkManager.getLocalIPAddresses()[0];
    	//System.out.println("********************initLevel="+initlevel);
    	//System.out.println("********************currentLevel="+currentLevel);
    	
    	int cpt=0;
    	while(cpt<MPR.size())
    	{
    		if (levelOf((String)MPR.elementAt(cpt))>=initlevel)
    		{
    			//System.out.println("I will remove "+MPR.elementAt(cpt)+" from MPR, because his level will be recalculated");
       			MPR.removeElementAt(cpt);
    		}
    		else cpt++;
    	}

    	// don't add my local address to the neighbors 
    	// don't add my neighbors addresses to N2 neighbors
    	// level = initlevel into the for
    	/*************************************
    	 *  CHANGE THE INITIAL VALUE (1) WITH THE ADEQUAT STARTED LEVEL
    	 ************************************/
    	//for (int level=initlevel ; level<HOPS_NUMBER_VALUE; level++)
    	for (int level=initlevel ; level<=HOPS_NUMBER_VALUE; level++)
    	{
    	//System.out.println("Level ="+level);
    	String[][] list = new String[300][500];
    	for (int i=0;i<100; i++)
        {
    		if (neighborsDirectories[i][0]!= null && !neighborsDirectories[i][0].equals("") && neighborsDirectoriesLevel[i]==level)
    			list[i][0]    =neighborsDirectories[i][0];
    		if (neighborsSD[i][0]!=null && !neighborsSD[i][0].equals("") && neighborsSDLevel[i]== level)
    			list[i+100][0]=neighborsSD[i][0];
    		if (neighborsNotCoveredSD[i][0]!=null && !neighborsNotCoveredSD[i][0].equals("") && neighborsNotCoveredSDLevel[i]== level)
    			list[i+200][0]=neighborsNotCoveredSD[i][0];
    		
        	for (int j=1; j<500; j++)
        	{
        		String ip="";
        		String iplevel = ""; 
        		if (neighborsDirectories[i][j]!= null && !neighborsDirectories[i][j].equals(""))
        		{
        		   ip = neighborsDirectories[i][j].substring(0, neighborsDirectories[i][j].indexOf(":"));
        		   iplevel = neighborsDirectories[i][j].substring(neighborsDirectories[i][j].indexOf(":")+1);
        		   if (iplevel.equals("1") && neighborsDirectoriesLevel[i]==level && !localAddress.equals(ip) && !isNeighbor(ip, level))
        		   {
        			list[i][j] = ip;
        		   }
        		}
        			
        		if (neighborsSD[i][j]!=null && !neighborsSD[i][j].equals(""))
        			{
        			ip = neighborsSD[i][j].substring(0, neighborsSD[i][j].indexOf(":"));
         		   	iplevel = neighborsSD[i][j].substring(neighborsSD[i][j].indexOf(":")+1);
        			if (iplevel.equals("1") && neighborsSDLevel[i] == level && !localAddress.equals(ip) && !isNeighbor(ip, level)) 
        				list[i+100][j]= ip;
        			}
        			
        		if (neighborsNotCoveredSD[i][j]!=null && !neighborsNotCoveredSD[i][j].equals(""))
        			{
        			ip = neighborsNotCoveredSD[i][j].substring(0, neighborsNotCoveredSD[i][j].indexOf(":"));
         		   	iplevel = neighborsNotCoveredSD[i][j].substring(neighborsNotCoveredSD[i][j].indexOf(":")+1);
        			if (iplevel.equals("1") && neighborsNotCoveredSDLevel[i]== level && !localAddress.equals(ip) && !isNeighbor(ip, level)) 
					list[i+200][j] = ip;
        			}
           	}
        }
    	
        String[] tmp = MPRSelection.getMPR(list, 300, 500);
        if (tmp.length<=0) 
        	{
        	logger.debug("MPR List updated ="+MPR.toString());
        		//System.out.println("**********************End Update MPR List****************");
        		return;
        	}
        for (int i=0; i<tmp.length; i++)
           if (!MPR.contains(tmp[i])) MPR.add(tmp[i]);
        MPRLevel = level;
        //currentLevel = level+1;
          currentLevel = level;
        
       	}
    	logger.debug("MPR List updated ="+MPR.toString());
    	//System.out.println("**********************End Update MPR List****************");
    }
    
    /**
     * 
     * @return
     */
    public String[] getMPR()
    {
    	String[] forReturn = new String[MPR.size()];
    	for (int i=0; i<MPR.size(); i++) 
    		forReturn[i]=(String)MPR.elementAt(i);
    	return forReturn;
    }
    /*
    public String[] getMPR()
    {
    	String localAddress = NetworkManager.getLocalIPAddresses()[0];
    	java.util.Vector forReturn = new java.util.Vector();
    			
    	// don't add my local address to the neighbors 
    	// don't add my neighbors addresses to N2 neighbors
    	System.out.println("HOPS_NUMBER_VALUE="+HOPS_NUMBER_VALUE);
    	for (int level=1; level<HOPS_NUMBER_VALUE; level++)
    	{
    	String[][] list = new String[300][500];
    	for (int i=0;i<100; i++)
        {
        	list[i][0]    =neighborsDirectories[i][0];
    	    list[i+100][0]=neighborsSD[i][0];
    	    list[i+200][0]=neighborsNotCoveredSD[i][0];
        	for (int j=1; j<500; j++)
        	{
        		if (neighborsDirectories[i][j]!= null && neighborsDirectoriesLevel[i]==level && !localAddress.equals(neighborsDirectories[i][j]) && !isNeighbor(neighborsDirectories[i][j], level)) 
        			list[i][j] = neighborsDirectories[i][j];
        			
        		if (neighborsSD[i][j]!=null && neighborsSDLevel[i]== level && !localAddress.equals(neighborsSD[i][j]) && !isNeighbor(neighborsSD[i][j], level)) 
        			list[i+100][j]= neighborsSD[i][j];
        			
        		if (neighborsNotCoveredSD[i][j]!=null && neighborsNotCoveredSDLevel[i]== level && !localAddress.equals(neighborsNotCoveredSD[i][j]) && !isNeighbor(neighborsNotCoveredSD[i][j], level)) 
        			list[i+200][j] = neighborsNotCoveredSD[i][j];
           	}
        }
    	
        String[] tmp = MPRSelection.getMPR(list, 300, 500);
        for (int i=0; i<tmp.length; i++)
          forReturn.add(tmp[i]);
        currentLevel = level;
       	}
    	
    	String[] MPR = new String[forReturn.size()];
    	for (int i=0; i<forReturn.size(); i++) 
    		MPR[i]=(String)forReturn.elementAt(i);
    	return MPR;
    }
    */
    private Thread NeighborsDeamon() {
        try
	    {
		Thread NeighborsDeamonThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
			    	while (start)
				    {
					byte[] receivedData = manager.getReceivedUDPData();
					if (receivedData!=null)
					{
					String data = new String(receivedData);
					
					data = data.trim();
					//System.out.println("data received ="+data);
					String counter = data.substring(0,data.indexOf("#"));
					//System.out.println("counter="+counter);
					int index = data.indexOf("#")+1;
					
					String hops = data.substring(index,data.indexOf("#", index));
					//System.out.println("hops="+hops);
					index = data.indexOf("#", index)+1;
					
					String MPRNumber = data.substring(index,data.indexOf("#", index));
					index = data.indexOf("#", index)+1;
					
					int MPRNum=0;
					try
					{
						MPRNum=Integer.valueOf(MPRNumber).intValue();
					}
					catch(NumberFormatException ex)
					{
						logger.error("NetworkCover: malformed number of MPR");	
					}
					
					Vector MPR = new Vector();
					for (int j=0;j<MPRNum;j++)
					{
						MPR.add(data.substring(index,data.indexOf("#", index)));
						index = data.indexOf("#", index)+1;
					}
					
					
					String messagePart = data.substring(index);
					
					String address = data.substring(index,data.indexOf("#", index));
					//System.out.println("address="+address);
					index = data.indexOf("#", index)+1;
					String status = data.substring(index,data.indexOf("#", index));
					//System.out.println("status="+status);
					index = data.indexOf("#", index)+1;
					String sdNumber = data.substring(index,data.indexOf("#", index));
					//System.out.println("sdNumber="+sdNumber);
					index = data.indexOf("#", index)+1;
					int sdNum=0;
					try
					{
						sdNum=Integer.valueOf(sdNumber).intValue();
					}
					catch(NumberFormatException ex)
					{
					logger.error("NetworkCover: malformed number of directories and SDs");	
					}
					String[] ipSDNeighbors = new String[sdNum];
					for (int j=0;j<sdNum;j++)
					{
						ipSDNeighbors[j]= data.substring(index,data.indexOf("#", index));
						index = data.indexOf("#", index)+1;
					}
					// SD not Covered
					String notCoveredSDNumber = data.substring(index,data.indexOf("#", index));
					index = data.indexOf("#", index)+1;
					int notCoveredSDNum=0;
					try
					{
						notCoveredSDNum=Integer.valueOf(notCoveredSDNumber).intValue();
					}
					catch(NumberFormatException ex)
					{
					logger.error("NetworkCover: malformed number of directories and SDs");	
					}
					String[] ipNotCoveredSDNeighbors = new String[notCoveredSDNum];
					for (int j=0;j<notCoveredSDNum;j++)
					{
						ipNotCoveredSDNeighbors[j]= data.substring(index,data.indexOf("#", index));
						index = data.indexOf("#", index)+1;
					}
					
					String dirNumber = data.substring(index,data.indexOf("#", index)); 
					index = data.indexOf("#", index)+1;
					int dirNum=0;
					try
					{
						dirNum=Integer.valueOf(dirNumber).intValue();
					}
					catch(NumberFormatException ex)
					{
					logger.error("NetworkCover: malformed number of directories and SDs");	
					}
					
					String[] ipDirNeighbors = new String[dirNum];
					for (int j=0;j<dirNum;j++)
					{
						ipDirNeighbors[j]= data.substring(index,data.indexOf("#", index));
						index = data.indexOf("#", index)+1;
					}
														
					if (address != null && !address.equals(NetworkManager.getLocalIPAddresses()[0]))
					    {
						    int intCounter=0;
							try
							{
								intCounter= Integer.valueOf(counter).intValue();
							}
							catch(NumberFormatException ex)
							{
								logger.error("NetworkCover: malformed number of counter");
							}
							if (status.equals("0")) addNotCoveredSDNeighbor(address, intCounter,sdNum,notCoveredSDNum,dirNum, ipSDNeighbors, ipNotCoveredSDNeighbors, ipDirNeighbors);
							else if (status.equals("1")) addSDNeighbor(address, intCounter,sdNum,notCoveredSDNum,dirNum, ipSDNeighbors, ipNotCoveredSDNeighbors, ipDirNeighbors);
							else if (status.equals("2")) addDirectoryNeighbor(address,intCounter, sdNum, notCoveredSDNum, dirNum, ipSDNeighbors, ipNotCoveredSDNeighbors, ipDirNeighbors);
							// foreward the message
							int intHops =1;
							try
							{
								intHops = Integer.valueOf(hops).intValue();
							}
							catch(NumberFormatException ex)
							{
								logger.error("NetworkCover: malformed number of hops");
							}
							//logger.debug("***************intHops="+intHops);
							if (MPR.contains(NetworkManager.getLocalIPAddresses()[0]) && intHops>intCounter+1)
							{
								MPR.remove(NetworkManager.getLocalIPAddresses()[0]);
								logger.debug("I forward a message hops="+intHops);
								//String message = address+"#"+String.valueOf(intHops-1)+"#"+status+"#"+sdNumber+"#"+dirNumber;
								//String message = data.substring(data.indexOf("#")+1);
								String mpr = MPR.size()+"";
								for (int m=0;m<MPR.size();m++)
									mpr = mpr + "#"+(String)MPR.elementAt(m);
								
								String message = String.valueOf(intCounter+1)+"#"+hops+"#"+mpr+"#"+messagePart;
								logger.debug("message forwarded="+message);
								byte[] data2send = message.getBytes();
								NetworkManager.broadcast(data2send, neighborsPort);
							}
					    }
					try
					{
						Thread.sleep(5);
					}
					catch(Exception ex)
					{
					logger.error("Error at NetworkCover.neighborsDemon:Thread.sleep");
				    }
					}
				    }
			    }	
			});	
		return NeighborsDeamonThread;
	    }
	catch (Exception ex)
	    {
		logger.error("at create NetworkManager.NeighborsDeamon Thread");
	    }
	return null;
    }

    private Thread NeighborsSender() {
        try
	    {
		Thread NeighborsSenderThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
			    	try
				    {
					while (start)
					    {
						String ipSDNeighbors   ="";
						String ipNotCoveredSDNeighbors   ="";
						String ipDirNeighbors  ="";
						
						int sdNeighborsNumber  = 0;
						int notCoveredSDNeighborsNumber = 0;
						int dirNeighborsNumber = 0;
						
						for (int i=0; i<100; i++)
						{
							if (neighborsSD[i][0]!= null && !neighborsSD[i][0].equals(""))
							{
								ipSDNeighbors=ipSDNeighbors+"#"+neighborsSD[i][0]+":"+neighborsSDLevel[i];
								sdNeighborsNumber++;
							}
						}
						
						for (int i=0; i<100; i++)
						{
							if (neighborsNotCoveredSD[i][0]!= null && !neighborsNotCoveredSD[i][0].equals(""))
							{
								ipNotCoveredSDNeighbors=ipNotCoveredSDNeighbors+"#"+neighborsNotCoveredSD[i][0]+":"+neighborsNotCoveredSDLevel[i];
								notCoveredSDNeighborsNumber++;
							}
						}
						
						for (int i=0; i<100;i++)
						{
							if (neighborsDirectories[i][0]!=null && !neighborsDirectories[i][0].equals(""))
							{
								ipDirNeighbors=ipDirNeighbors+"#"+neighborsDirectories[i][0]+":"+neighborsDirectoriesLevel[i];
								dirNeighborsNumber++;
							}
						}
						
						//String message = NetworkManager.getLocalIPAddresses()[0]+"#"+String.valueOf(HOPS_NUMBER_VALUE)+"#"+messageType+"#"+sdNeighborsNumber+"#"+dirNeighborsNumber;
						// get MPR List
						
						String[] MPRList = getMPR();
						String mpr = MPRList.length+"";
						for (int m=0; m<MPRList.length ; m++)
							mpr=mpr+"#"+MPRList[m];
						
						String message = "0#"+String.valueOf(currentLevel)+"#"+mpr+"#"+NetworkManager.getLocalIPAddresses()[0]+"#"+messageType+"#"+sdNeighborsNumber;
						if (sdNeighborsNumber>0) message=message+ipSDNeighbors;
						message=message+"#"+notCoveredSDNeighborsNumber;
						if (notCoveredSDNeighborsNumber>0) message=message+ipNotCoveredSDNeighbors;
						message=message+"#"+dirNeighborsNumber;
						if (dirNeighborsNumber>0) message=message+ipDirNeighbors;
						message = message+"#";
						byte[] data = message.getBytes();
						NetworkManager.broadcast(data, neighborsPort);
						logger.debug("message sent="+message);
						Thread.sleep(NEIGHBORS_ADVERTISEMENT_VALUE);
					    }
				    }
				catch(Exception ex)
				    {
					logger.error("at NetworkCover.NeighborsSender Thread");
				    }
			    }
			});
			return NeighborsSenderThread;
	    }
	catch (Exception ex)
	    {
		logger.error("at create NetworkCover.NeighborsSender Thread");
	    }
	return null;
    }

    private Thread TTLUpdate() {
        try
	    {
		Thread TTLUpdateThread = 
		    new Thread( new Runnable() 
			{
			    public void run() 
			    {
				while (start)
				    {
					try
					    {
						Thread.sleep(NEIGHBORS_LISTS_TTL_UPDATE_VALUE);
						for (int index=0;index<100;index++)
						    {
							if (TTL4SD[index]>0) 
							    {
								TTL4SD[index]-=NEIGHBORS_LISTS_TTL_UPDATE_VALUE;
								//System.out.println("SD "+index+" ttl="+TTL4SD[index]);
								if (TTL4SD[index] == 0) 
								    {
									logger.debug("SD "+neighborsSD[index][0]+" is removed");
									for (int i=0;i<500;i++)
										neighborsSD[index][i]="";
									cptSD--;
									//if (MPR.contains(neighborsSD[index][0])) 
										updateMPRlist(neighborsSDLevel[index]-1);
								    }
							    }
							if (TTL4NotCoveredSD[index]>0) 
						    {
							TTL4NotCoveredSD[index]-=NEIGHBORS_LISTS_TTL_UPDATE_VALUE;
							//System.out.println("NotCoveredSD "+index+" ttl="+TTL4NotCoveredSD[index]);
							if (TTL4NotCoveredSD[index] == 0) 
							    {
								logger.debug("not Covered SD "+neighborsNotCoveredSD[index][0]+" is removed");
								for (int i=0;i<500;i++)
									neighborsNotCoveredSD[index][i]="";
								//if (MPR.contains(neighborsNotCoveredSD[index][0])) 
									updateMPRlist(neighborsNotCoveredSDLevel[index]-1);
								cptNotCoveredSD--;
							    }
						    }
							if (TTL4Dir[index]>0) 
							    {
								TTL4Dir[index]-=NEIGHBORS_LISTS_TTL_UPDATE_VALUE;
								if (TTL4Dir[index] == 0) 
								    {
									logger.debug("Dir "+neighborsDirectories[index][0]+ "  level="+neighborsDirectoriesLevel[index]+" is removed");
									cptDirectory--;
									for (int i=0;i<500;i++)
										neighborsDirectories[index][i]="";
									//if (MPR.contains(neighborsDirectories[index][0])) 
										updateMPRlist(neighborsDirectoriesLevel[index]-1);
								    }
							    }
						    }
					    }
					catch(Exception ex)
					    {
							logger.error("at NetworkCover.TTLUpdate Thread");
					    }
				    }
			    }
			});
		return TTLUpdateThread;
	    }
	catch (Exception ex)
	    {
		logger.error("create NetworkCover.TTLUpdate Thread");
	    }
	return null;
    }

    public static void main(String[] arg)
    {
	/*
	try
	    {
		Thread.sleep(2000);
	    }
	catch(Exception ex)
	    {
		System.out.println("Error at NetworkCover.main.sleep");
	    }

	System.out.println("app.getNeighborsNumber()="+app.getNeighborsNumber());
	String[] address = app.getNeighborsAddresses();
	
		if (address !=null && address.length>0)
	    for (int i=0; i< address.length;i++)
		{
		    System.out.println("address"+i+"="+address[i]);
		    System.out.println("TTL="+app.TTL4SD[i]);
		}
	
				try
	    {
		Thread.sleep(4000);
	    }
	catch(Exception ex)
	    {
		System.out.println("Error at NetworkCover.main.sleep");
	    }
	System.out.println("app.getNeighborsNumber()="+app.getNeighborsNumber());
	address = app.getNeighborsAddresses();
	if (address !=null && address.length>0)
	    for (int i=0; i< address.length;i++)
		{
		    System.out.println("address"+i+"="+address[i]);
		    System.out.println("TTL="+app.TTL4SD[i]);
		}
		
	try
	{
		Thread.sleep(20000);
	}
	catch(Exception ex)
	{
		System.out.println("Error at NetworkCover.main.sleep");
	}

	//app.updateMPRlist(1);
	String[] MPR = app.getMPR();
	if (MPR!=null)
	{
		System.out.println("all level MPR List : ");
		for (int i=0;i<MPR.length;i++)
			System.out.println(MPR[i]+" ");
	}
	
		//app.stop();
		 * 
		 */
    }

} 
